// bezier07View.cpp : implementation of the CBezierView class
//

#include "stdafx.h"
#include "SRAF.h"
#include "Bezier.h"
#include <stdlib.h>

#include "BezierSet.h"
#include "BezierDoc.h"
#include "BezierView.h"
//#include "OcxDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBezierView

IMPLEMENT_DYNCREATE(CBezierView, CScrollView)

BEGIN_MESSAGE_MAP(CBezierView, CScrollView)
	//{{AFX_MSG_MAP(CBezierView)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_RBUTTONDOWN()
	ON_COMMAND(ID_EDIT_POINT, OnEditPoint)
	ON_COMMAND(ID_ADD_POINT, OnAddPoint)
	ON_COMMAND(ID_DEL_POINT, OnDelPoint)
	ON_COMMAND(ID_VIEW_CURVE, OnViewCurve)
	ON_UPDATE_COMMAND_UI(ID_ADD_POINT, OnUpdateAddPoint)
	ON_UPDATE_COMMAND_UI(ID_DEL_POINT, OnUpdateDelPoint)
	ON_UPDATE_COMMAND_UI(ID_EDIT_POINT, OnUpdateEditPoint)
	ON_UPDATE_COMMAND_UI(ID_VIEW_CURVE, OnUpdateViewCurve)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBezierView construction/destruction

CBezierView::CBezierView() : rect(50,50,500,500)
{
	m_bCaptured = FALSE;
	r = g = b = 40;
}

CBezierView::~CBezierView()
{
}

BOOL CBezierView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CBezierView drawing

void CBezierView::OnDraw(CDC* pDC)
{
	//CBezierDoc* pDoc = GetDocument();
	//ASSERT_VALID(pDoc);

	pDC->SetMapMode(MM_TEXT);

	//CBrush br(RGB(255,0,0));
	//pDC->FillRgn(&GetBezier()->m_rgn, &br);

	GetBezier()->Draw(pDC);
	//GetBezier()->Handles(pDC);
	
	/*CString texto;
	texto.Format("Ver:4, (%2.4f,%2.4f), (%2.4f,%2.4f)",curve->ux.max, curve->uy.max,curve->ux.min, curve->uy.min);
	pDC->TextOut(0,0,texto); 
	texto.Format("Ver:4, ax=%.2f, bx=%.2f, cx=%.2f, ay=%.2f, by=%.2f, cy=%.2f)",curve->ux.a, curve->ux.b,curve->ux.c,curve->uy.a,curve->uy.b,curve->uy.c);
	pDC->TextOut(0,20,texto); */
}

/////////////////////////////////////////////////////////////////////////////
// CBezierView diagnostics

#ifdef _DEBUG
void CBezierView::AssertValid() const
{
	CView::AssertValid();
}

void CBezierView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CBezierDoc* CBezierView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CBezierDoc)));
	return (CBezierDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CBezierView message handlers

void CBezierView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if (GetBezier()->OnMouseClick(point)) {
		SetCapture();
		m_bCaptured = TRUE;
		InvalidateRgn(&GetBezier()->m_rgn);
		::SetCursor(::LoadCursor(NULL,IDC_CROSS));
		GetDocument()->SetModifiedFlag();
	}
}

void CBezierView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if (m_bCaptured) {
		::ReleaseCapture();
		GetBezier()->OnReleaseCapture();	
		m_bCaptured = FALSE;
	}
}

void CBezierView::OnMouseMove(UINT nFlags, CPoint point) 
{
  	if (m_bCaptured) {
		GetBezier()->OnMoveCaptured(point);
		//Invalidate();
		InvalidateRgn(&GetBezier()->m_rgn);
	}	
}

void CBezierView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	Invalidate();
	
	CView::OnRButtonDown(nFlags, point);
}

void CBezierView::OnEditPoint() 
{
	GetBezier()->SetEditMode();
	Invalidate();
	//::SetCursor(AfxGetApp()->LoadCursor(IDC_EDIT_POINT));	
	//InvalidateRgn(&GetBezier()->m_rgn);
}

void CBezierView::OnAddPoint() 
{
	GetBezier()->SetAddMode();
	Invalidate();
}

void CBezierView::OnDelPoint() 
{
	GetBezier()->SetDelMode();
	Invalidate();
}

void CBezierView::OnViewCurve() 
{
	GetBezier()->SetViewMode();
	Invalidate();
}

void CBezierView::OnInitialUpdate() 
{
	CScrollView::OnInitialUpdate();
	CSize sizeTotal(20000, 30000); //20 by 30 cm
	CSize sizePage(sizeTotal.cx / 2, sizeTotal.cy / 2);
	CSize sizeLine(sizeTotal.cx / 50, sizeTotal.cy / 50);
	SetScrollSizes(MM_HIMETRIC, sizeTotal, sizePage, sizeLine);
}

void CBezierView::OnUpdateAddPoint(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(GetBezier()->IsAddMode());
}

void CBezierView::OnUpdateDelPoint(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(GetBezier()->IsDelMode());
}

void CBezierView::OnUpdateEditPoint(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(GetBezier()->IsEditMode());
}

void CBezierView::OnUpdateViewCurve(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(GetBezier()->IsViewMode());
}

