//#include "c:\MSDEV\MFC\src\stdafx.h"
#include "stdafx.h"
#include "Dib.h"

/////////////////////////////////////////////////////////////////////////////
// CDib

IMPLEMENT_SERIAL(CDib, CObject, 0);

CDib::CDib()
{
	m_pBMI = NULL;
	m_pBits = NULL;
	m_nUbicBMI = m_nUbicBits = NO_UBIC;
	m_hFile = NULL;
	m_hSeccion = NULL;
	m_hPaleta = NULL;
	m_nMaxFila = 0;
	m_nMaxColumna = 0;
}

CDib::~CDib()
{
	Vaciar();
}

BOOL CDib::Crear(const CSize& dim, int bpp/*bit per pixel*/)
{
	return Crear(dim.cx, dim.cy, bpp);
}

BOOL CDib::Crear(int ancho, int alto, int bpp/*bit per pixel*/)
{
	Vaciar();
	if (CrearBMI(ancho, alto, bpp)) {
		m_pBits = (BYTE*) new char[ObtDimBits()];
		if (m_pBits == NULL) return FALSE;
		else m_nUbicBits = CRT_HEAP;
	}
	return TRUE;
}

HBITMAP CDib::CrearSeccion(CSize dim, int bpp, CDC* pDC)
{
	Vaciar();
	if (CrearBMI(dim.cx, dim.cy, bpp)) {
		m_hSeccion = ::CreateDIBSection(pDC->GetSafeHdc(), m_pBMI,
			DIB_RGB_COLORS,	(LPVOID*) &m_pBits, NULL, 0);
		ASSERT(m_pBits != NULL);
		return m_hSeccion;
	}
	return NULL;
}

BOOL CDib::Cargar(const char* path, UINT nOpenFlags)
{
	CFile file;

	Vaciar();
	if (file.Open(path, nOpenFlags))
		return Cargar(&file);
	else
		return FALSE;
}

BOOL CDib::Cargar(CFile *pFile)
{
	int nCount, nSize;
	BITMAPFILEHEADER bmfh;

	Vaciar();
	
	TRY {
		nCount = pFile->Read((LPVOID) &bmfh, sizeof(BITMAPFILEHEADER));
		if(nCount != sizeof(BITMAPFILEHEADER)) {
			AfxMessageBox("Error de lectura 1");
			return FALSE;
		}
		if(bmfh.bfType != 0x4d42) {
			AfxMessageBox("Archivo CDib invalido");
			return FALSE;
		}
		nSize = bmfh.bfOffBits - sizeof(BITMAPFILEHEADER);
		m_pBMI = (BITMAPINFO*) new char[nSize];
		m_nUbicBMI = m_nUbicBits = CRT_HEAP;
		nCount = pFile->Read(m_pBMI, nSize); // info hdr & color table

		m_pBits = (LPBYTE) new char[ObtDimBits()];
		nCount = pFile->Read(m_pBits, ObtDimBits()); // image only
	}
	CATCH (CException, e) {
		AfxMessageBox("Error de lectura 1");
		return FALSE;
	}
	END_CATCH

	m_nMaxFila = m_pBMI->bmiHeader.biHeight - 1;
	m_nMaxColumna = (m_pBMI->bmiHeader.biWidth + 3) & ~3;
	
	return TRUE;
}

BOOL CDib::Mapear(const char* path, BOOL compartir /*=FALSE*/)
{
	// if we open the same file twice, Windows treats it as 2 separate files
	HANDLE hFile = ::CreateFile(path, GENERIC_READ, compartir ? FILE_SHARE_READ : 0,
		NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	ASSERT(hFile);
	DWORD dwFileSize = ::GetFileSize(hFile, NULL);
	HANDLE hMap = ::CreateFileMapping(hFile, NULL, PAGE_READONLY, 0, 0, NULL);
	//DWORD dwErr = ::GetLastError();
	if(hMap == NULL) {
		AfxMessageBox("Archivo CDib vacio");
		return FALSE;
	}
	LPVOID lpvFile = ::MapViewOfFile(hMap, FILE_MAP_READ, 0, 0, 0); // map whole file
	if(((LPBITMAPFILEHEADER) lpvFile)->bfType != 0x4d42) {
		AfxMessageBox("Archivo CDib invalido");
		return FALSE;
	}
	EngancharMemoria((LPBYTE) lpvFile + sizeof(BITMAPFILEHEADER));
	m_lpvFile = lpvFile;
	m_hFile = hFile;
	m_hMap = hMap;
	return TRUE;
}

void CDib::EngancharMemoria(void* pvMem, BOOL eliminar /*=FALSE*/, HGLOBAL hGlobal /*=FALSE*/)
{
	Vaciar();
		m_hGlobal = hGlobal;
	if(eliminar == FALSE) {
		m_nUbicBMI = NO_UBIC;
	}
	else {
		m_nUbicBMI = ((hGlobal == NULL) ? CRT_HEAP : GLOBAL_HEAP);
	}
	m_pBMI = (BITMAPINFO*) pvMem;
	m_pBits = (BYTE*) m_pBMI->bmiColors + sizeof(RGBQUAD) * ObtDimPaleta();

	m_nMaxFila = m_pBMI->bmiHeader.biHeight - 1;
	m_nMaxColumna = (m_pBMI->bmiHeader.biWidth + 3) & ~3;
}
	
BOOL CDib::CopiarBits()
{
	return FALSE;
}

BOOL CDib::CopiarAMapFile(const char* path)
{
	// copies DIB to a new file, releases prior pointers
	// if you previously used CreateSection, the HBITMAP will be NULL (and unusable)
	BITMAPFILEHEADER bmfh;
	int dimPaleta = ObtDimPaleta(), dimBits = ObtDimBits();

	bmfh.bfType = 0x4d42;  // 'BM'
	bmfh.bfSize = dimBits + sizeof(BITMAPINFOHEADER) +
			sizeof(RGBQUAD) * dimPaleta + sizeof(BITMAPFILEHEADER);
	// meaning of bfSize open to interpretation
	bmfh.bfReserved1 = bmfh.bfReserved2 = 0;
	bmfh.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) +
			sizeof(RGBQUAD) * dimPaleta;	
	HANDLE hFile = ::CreateFile(path, GENERIC_WRITE | GENERIC_READ, 0, NULL,
		CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	ASSERT(hFile);
	int nSize =  sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) +
				sizeof(RGBQUAD) * dimPaleta +  dimBits;
	HANDLE hMap = ::CreateFileMapping(hFile, NULL, PAGE_READWRITE, 0, nSize, NULL);
	DWORD dwErr = ::GetLastError();
	ASSERT(hMap);
	LPVOID lpvFile = ::MapViewOfFile(hMap, FILE_MAP_WRITE, 0, 0, 0); // map whole file
	ASSERT(lpvFile);
	LPBYTE lpbCurrent = (LPBYTE) lpvFile;
	memcpy(lpbCurrent, &bmfh, sizeof(BITMAPFILEHEADER)); // file header
	lpbCurrent += sizeof(BITMAPFILEHEADER);
	LPBITMAPINFOHEADER lpBMIH = (LPBITMAPINFOHEADER) lpbCurrent;
	memcpy(lpbCurrent, m_pBMI,
		sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * dimPaleta); // info
	lpbCurrent += sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * dimPaleta;
	memcpy(lpbCurrent, m_pBits, dimBits); // bit image
	DWORD dwSizeImage = dimBits;
	Vaciar();
	//m_dwSizeImage = dimBits;	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	m_nUbicBMI = m_nUbicBits = NO_UBIC;
	m_pBMI = (BITMAPINFO*) lpBMIH;		//ojo con conv.
	m_pBits = lpbCurrent;
	m_hFile = hFile;
	m_hMap = hMap;
	m_lpvFile = lpvFile;

	m_nMaxFila = m_pBMI->bmiHeader.biHeight - 1;
	m_nMaxColumna = (m_pBMI->bmiHeader.biWidth + 3) & ~3;

	return TRUE;
}

BOOL CDib::Copiar(CFile* pFile)
{
	BITMAPFILEHEADER bmfh;
	DWORD nDim = ObtDimBits();
	bmfh.bfType = 0x4d42;  // 'BM'
	int nSize =  sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * ObtDimPaleta() + nDim;
	bmfh.bfSize = nSize + sizeof(BITMAPFILEHEADER);
	// meaning of bfSize open to interpretation (bytes, words, dwords?) -- we won't use it
	bmfh.bfReserved1 = bmfh.bfReserved2 = 0;
	bmfh.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) +
			sizeof(RGBQUAD) * ObtDimPaleta();	
	TRY {
		pFile->Write((LPVOID) &bmfh, sizeof(BITMAPFILEHEADER));
		pFile->Write((LPVOID) m_pBMI, nSize - nDim);
		pFile->Write((LPVOID) m_pBits, nDim);
	}
	CATCH (CException, e) {
		AfxMessageBox("Error de escritura");
		return FALSE;
	}
	END_CATCH
	return TRUE;
}

void CDib::Serialize(CArchive& ar)
{
	ar.Flush();
	if(ar.IsStoring()) {
		Copiar(ar.GetFile());
	}
	else {
		Cargar(ar.GetFile());
	}
}

BOOL CDib::AsgPaleta()
{
	int colores = ObtDimPaleta();

	if(m_pBMI == NULL || colores == 0) return FALSE;
	if(m_hPaleta != NULL) ::DeleteObject(m_hPaleta);
	
	LPLOGPALETTE pLogPal = (LPLOGPALETTE) new char[2 * sizeof(WORD) +
		colores * sizeof(PALETTEENTRY)];
	pLogPal->palVersion = 0x300;
	pLogPal->palNumEntries = colores;
	LPRGBQUAD pDibQuad = (LPRGBQUAD) colores;
	
	for(int i = 0; i < colores; i++) {
		pLogPal->palPalEntry[i].peRed = pDibQuad->rgbRed;
		pLogPal->palPalEntry[i].peGreen = pDibQuad->rgbGreen;
		pLogPal->palPalEntry[i].peBlue = pDibQuad->rgbBlue;
		pLogPal->palPalEntry[i].peFlags = 0;
		pDibQuad++;
	}

	m_hPaleta = ::CreatePalette(pLogPal);
	delete pLogPal;
	return TRUE;
}

BOOL CDib::AsgPaletaDelSistema(CDC* pDC)
{
	// si el CDib no tiene paleta no hay nada que hacer
	if(m_pBMI == NULL || ObtDimPaleta() != 0) return FALSE;
	HDC hDC = pDC->GetSafeHdc();

	if(!::GetDeviceCaps(hDC, RASTERCAPS) & RC_PALETTE) return FALSE;
	
	int nSysColors = ::GetDeviceCaps(hDC, NUMCOLORS);
	int nPalEntries = ::GetDeviceCaps(hDC, SIZEPALETTE);
	
	ASSERT(nPalEntries <= 256);
	
	int nEntries = (nPalEntries == 0) ? nSysColors : nPalEntries;
	LPLOGPALETTE pLogPal = (LPLOGPALETTE) new char[2 * sizeof(WORD) +
		nEntries * sizeof(PALETTEENTRY)];
	pLogPal->palVersion = 0x300;
	pLogPal->palNumEntries = nEntries;
	::GetSystemPaletteEntries(hDC, 0, nEntries,
		(LPPALETTEENTRY) ((LPBYTE) pLogPal + 2 * sizeof(WORD)));
	m_hPaleta = ::CreatePalette(pLogPal);
	delete pLogPal;
	return TRUE;
}

// helper functions
int CDib::ObtDimPaleta(int nBitCount /*=0*/)
{
	if (m_pBMI == NULL) return 0;

	if (nBitCount == 0) nBitCount = m_pBMI->bmiHeader.biBitCount;

	ASSERT(m_pBMI->bmiHeader.biClrUsed <= 256); 

	if(m_pBMI->bmiHeader.biClrUsed == 0) {
		switch(nBitCount) {
			case 1:
				return 2;
			case 4:
				return 16;
			case 8:
				return 256;
			case 16:
			case 24:
			case 32:
				return 0;
			default:
				ASSERT(FALSE);
				return 0;
		}
	}
	else {
		return m_pBMI->bmiHeader.biClrUsed;
	}
}

CSize CDib::ObtDim()
{
	if(m_pBMI == NULL) return CSize(0, 0);
	return CSize((int) m_pBMI->bmiHeader.biWidth, (int) m_pBMI->bmiHeader.biHeight);
}

DWORD CDib::ObtDimBits()
{
	DWORD medida = m_pBMI->bmiHeader.biSizeImage;
	if(medida == 0) {
		medida = ((DWORD) m_pBMI->bmiHeader.biWidth * m_pBMI->bmiHeader.biBitCount) / 32;
		if(((DWORD) m_pBMI->bmiHeader.biWidth * m_pBMI->bmiHeader.biBitCount) % 32) {
			medida++;
		}
		medida *= 4 * m_pBMI->bmiHeader.biHeight; //sin comprimir
	}
	return medida;
}

int CDib::ObtBitsPorPixel()
{
	return (m_pBMI) ? m_pBMI->bmiHeader.biBitCount:0;
}

void CDib::Vaciar()
{
	// si no hay BMI seguro esta vacio
	if (m_pBMI == NULL) return;
	
	if(m_hFile != NULL) {
		::UnmapViewOfFile(m_lpvFile);
		::CloseHandle(m_hMap);
		::CloseHandle(m_hFile);
		m_hFile = NULL;
	}
	if(m_nUbicBMI == CRT_HEAP) {
		delete [] m_pBMI;
	}
	else if(m_nUbicBMI == GLOBAL_HEAP) {
		::GlobalUnlock(m_hGlobal);
		::GlobalFree(m_hGlobal);
	}
	if(m_nUbicBits == CRT_HEAP) delete[] m_pBits;
	if(m_hPaleta != NULL) ::DeleteObject(m_hPaleta);
	if(m_hSeccion != NULL) ::DeleteObject(m_hSeccion);
	m_nUbicBMI = m_nUbicBits = NO_UBIC;
	m_hGlobal = NULL;
	m_pBMI = NULL;
	m_pBits = NULL;
	m_lpvFile = NULL;
	m_hMap = NULL;
	m_hFile = NULL;
	m_hSeccion = NULL;
	m_hPaleta = NULL;
	m_nMaxFila = 0;
	m_nMaxColumna = 0;
}

BOOL CDib::CrearBMI(int ancho, int alto, int bpp)
{
	int colores = 256; //ObtDimPaleta(bpp);

	m_pBMI = (BITMAPINFO*) 
		new char[sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * colores];
	
	if (m_pBMI == NULL) return FALSE;
	
	m_nUbicBMI = CRT_HEAP;

	m_pBMI->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	m_pBMI->bmiHeader.biWidth = ancho;
	m_pBMI->bmiHeader.biHeight = alto;
	m_pBMI->bmiHeader.biPlanes = 1;
	m_pBMI->bmiHeader.biBitCount = bpp;
	m_pBMI->bmiHeader.biCompression = BI_RGB;
	m_pBMI->bmiHeader.biSizeImage = 0;
	m_pBMI->bmiHeader.biXPelsPerMeter = 0;
	m_pBMI->bmiHeader.biYPelsPerMeter = 0;
	m_pBMI->bmiHeader.biClrUsed = colores;
	m_pBMI->bmiHeader.biClrImportant = colores;

	m_nMaxFila = alto - 1;
	m_nMaxColumna = (ancho + 3) & ~3;

	RGBQUAD* prgb = (LPRGBQUAD)(((BYTE*)(m_pBMI)) + sizeof(BITMAPINFOHEADER));
    for (int i = 0; i < colores; i++) {
		prgb->rgbBlue = prgb->rgbGreen = prgb->rgbRed = (BYTE) i;
        prgb->rgbReserved = 0;
        prgb++;
    }
	return TRUE;
}

BOOL CDib::Dibujar(CDC* pDC, CPoint origin, CSize size)
{
    if (m_pBMI == NULL) return FALSE;
    if (m_hPaleta != NULL) {
      ::SelectPalette(pDC->GetSafeHdc(), m_hPaleta, TRUE);
    }
    ::StretchDIBits(pDC->GetSafeHdc(), origin.x, origin.y, size.cx, size.cy,
        0, 0, m_pBMI->bmiHeader.biWidth, m_pBMI->bmiHeader.biHeight,
        m_pBits, (LPBITMAPINFO) m_pBMI, DIB_RGB_COLORS, SRCCOPY);
    return TRUE;
}


void CDib::Crear(const CDib& bmp)
{
	EngancharMemoria(bmp.m_pBits);
}

#ifdef _DEBUG
void CDib::Dump( CDumpContext& dc ) const
{
    // call base class function first
    CObject::Dump( dc );

    // now do the stuff for our specific class
    dc << "m_pBMI: " << m_pBMI << "\n"
        << "m_pBits: " << m_pBits << "\n"
		<< "m_pBMI->bmiHeader.biWidth: " << m_pBMI->bmiHeader.biWidth << "\n"
		<< "m_pBMI->bmiHeader.biHeight: " << m_pBMI->bmiHeader.biHeight << "\n"
		<< "m_pBMI->bmiHeader.biWidth: " << m_pBMI->bmiHeader.biWidth << "\n"
		<< "m_nMaxFila: " << m_nMaxFila << "\n"
		<< "m_nMaxColumna: " << m_nMaxColumna << "\n";
}
#endif
